package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import org.eclipse.jdt.internal.compiler.ast.FakedTrackingVariable;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.Scope;

public class IteratorForReporting implements Iterator<FakedTrackingVariable> {
  private final Set<FakedTrackingVariable> varSet;
  
  private final Scope scope;
  
  private final boolean atExit;
  
  private Stage stage;
  
  private Iterator<FakedTrackingVariable> iterator;
  
  private FakedTrackingVariable next;
  
  public IteratorForReporting(List<FakedTrackingVariable> variables, Scope scope, boolean atExit) {
    this.varSet = new HashSet<>(variables);
    this.scope = scope;
    this.atExit = atExit;
    setUpForStage(Stage.OuterLess);
  }
  
  public boolean hasNext() {
    switch (this.stage) {
      case OuterLess:
        while (this.iterator.hasNext()) {
          FakedTrackingVariable trackingVar = this.iterator.next();
          if (trackingVar.outerTracker == null)
            return found(trackingVar); 
        } 
        setUpForStage(Stage.InnerOfProcessed);
      case InnerOfProcessed:
        while (this.iterator.hasNext()) {
          FakedTrackingVariable trackingVar = this.iterator.next();
          FakedTrackingVariable outer = trackingVar.outerTracker;
          if (outer.binding.declaringScope == this.scope && !this.varSet.contains(outer))
            return found(trackingVar); 
        } 
        setUpForStage(Stage.InnerOfNotEnclosing);
      case InnerOfNotEnclosing:
        while (this.iterator.hasNext()) {
          FakedTrackingVariable trackingVar = this.iterator.next();
          FakedTrackingVariable outer = trackingVar.outerTracker;
          if (!this.varSet.contains(outer)) {
            BlockScope blockScope = outer.binding.declaringScope;
            Scope currentScope = this.scope;
            do {
              if (!(currentScope = currentScope.parent instanceof BlockScope))
                return found(trackingVar); 
            } while (blockScope != currentScope);
            break;
          } 
        } 
        setUpForStage(Stage.AtExit);
      case null:
        if (this.atExit && this.iterator.hasNext())
          return found(this.iterator.next()); 
        return false;
    } 
    throw new IllegalStateException("Unexpected Stage " + this.stage);
  }
  
  private boolean found(FakedTrackingVariable trackingVar) {
    this.iterator.remove();
    this.next = trackingVar;
    return true;
  }
  
  private void setUpForStage(Stage nextStage) {
    this.iterator = this.varSet.iterator();
    this.stage = nextStage;
  }
  
  public FakedTrackingVariable next() {
    return this.next;
  }
  
  public void remove() {
    throw new UnsupportedOperationException();
  }
}
